
INSTALLED_APPS +=(
    'bootstrapform',
    'django.contrib.staticfiles',
    'django_tables2',
    'background_task',
    'django_tables2_reports',
    'django_excel_to_model',
)


TEMPLATE_CONTEXT_PROCESSORS += (
    "django.core.context_processors.request",
    'django.core.context_processors.static',
)



from django.views.decorators.csrf import csrf_exempt
from django.shortcuts import render
from django_excel_to_model.management.commands.import_excel_according_to_model import ExcelFileFromClassImporter
from django_excel_to_model.tasks import import_excel
from forms import ExcelImportTaskForm


__author__ = 'weijia'


@csrf_exempt
def upload_file(request):
    if request.method == 'POST':
        form = ExcelImportTaskForm(request.POST, request.FILES)
        if form.is_valid():
            # file is saved
            instance = form.save()
            # return HttpResponseRedirect('/excel_background_importer/')
            instance.next_process_line_numbered_from_1 = instance.header_row_numbered_from_1+1
            instance.save()
            if form.cleaned_data["is_import_now"]:
                if form.cleaned_data["is_clean_before_import"]:
                    instance.content.model_class().objects.all().delete()
                e = ExcelFileFromClassImporter(instance.content.model_class(), instance.spreadsheet_numbered_from_1)
                first_import_row_numbered_from_1 = instance.header_row_numbered_from_1+1
                import_cnt = 999999999
                e.import_excel(instance.excel_file.path, instance.header_row_numbered_from_1,
                               first_import_row_numbered_from_1, import_cnt)
                instance.is_completed = True
                instance.save()
            else:
                import_excel()
    else:
        form = ExcelImportTaskForm()
    return render(request, 'django_excel_to_model/upload.html',
                  {'file_form': form})



import os
# from excel_reader.openpyxl_reader import ExcelFile

from django.core.management.base import BaseCommand, CommandError
from django_excel_to_model.field_tools import get_valid_excel_field_name, get_target_field_name
from django_excel_to_model.reader import ExcelFile
from ufs_tools.folder_file_processor import FolderFileProcessor
import re
import random


def rename_if_field_name_starts_with_number(model_attribute_name):
    if not (re.match("^[0-9].+", model_attribute_name) is None):
        model_attribute_name = "n" + model_attribute_name
    return model_attribute_name


class ModelCreator(object):
    reserved_keywords = ["type", "class", "for", "in", "while"]
    MAX_RECORD_LENGTH = 64000/4  # for utf8 there will be multiple byte for one char
    MAX_FIELD_NAME_LENGTH = (64-5)

    def __init__(self, full_path, header_row_start_from_0=0, sheet_index_numbered_from_0=0, class_name="YourClass"):
        super(ModelCreator, self).__init__()
        # self.mapping = mapping_dict
        # self.attr_order = attr_order_list
        self.field_num = 0
        self.indents = u" " * 4
        self.attr_list = []
        self.mapping_dict = {}
        self.model_code_lines = []
        self.invalid_field_name = u"__invalid"
        self.attr_list_code_lines = []
        self.mapping_code_lines = []
        excel_file = ExcelFile(full_path)
        self.worksheet = excel_file.get_sheet(sheet_index_numbered_from_0)
        self.header = self.worksheet.get_header_raw(header_row_start_from_0)
        self.class_name = class_name
        self.field_len_definition = 256

    def create_model(self, data_start_row=1):
        code = self.get_mapping_and_attr()
        attr_order = None
        mapping = None
        exec code

        res = [u"", u"", u"# @python_2_unicode_compatible",
               u"class %s(models.Model):" % self.class_name]
        for row_data_dict in self.worksheet.enumerate(data_start_row):
            for key in attr_order:
                if key == "":
                    continue
                help_text = get_valid_excel_field_name(key)

                first_part = u'%s%s = models.CharField(max_length=%s, help_text=_("""' % \
                             (self.indents, mapping[key], self.field_len_definition)
                declaration = first_part + help_text + u'"""), null=True, blank=True, verbose_name="""'+help_text+'""")'
                res.append(declaration)
            break
        res.append(self.indents + u'data_import_id = models.CharField(max_length=TEXT_LENGTH_128, '
                                  u'help_text=_("Id for this import"), null=True, blank=True)')
        res.append(self.indents + u'')
        res.append(self.indents + u'# def __str__(self):')
        res.append(self.indents + u'# ' + self.indents + u'return "%s" % self.name')
        self.model_code_lines = res
        return u"\n".join(res)

    def get_default_field_len_definition(self):
        max_len_for_each_field = int(self.MAX_RECORD_LENGTH / self.field_num)
        if max_len_for_each_field == 0:
            raise "Too many fields"
        max_target_len = 1024
        while (max_target_len & max_len_for_each_field) == 0:
            max_target_len >>= 1
        field_len_definition = u"TEXT_LENGTH_" + str(max_target_len)
        return field_len_definition

    def create_mapping(self, target_mapping_name="mapping"):
        self.mapping_code_lines = ["%s = {" % target_mapping_name, ]

        self.create_field_title_to_attr_name_mapping()

        for key in self.mapping_dict:
            self.mapping_code_lines.append(self.indents + 'u"""%s""": u"%s",' % (key, self.mapping_dict[key]))
            self.field_num += 1

        self.mapping_code_lines.append("}")

        self.field_len_definition = self.get_default_field_len_definition()

    def create_field_title_to_attr_name_mapping(self):
        for col in self.header:
            if col == "":
                self.mapping_dict[u""] = self.invalid_field_name
            else:
                field_title_str = get_valid_excel_field_name(col)
                model_attribute_name = self.get_attribute_name(col)
                self.mapping_dict[field_title_str] = model_attribute_name

    def get_attribute_name(self, col):
        model_attribute_name = get_target_field_name(col).lower()
        model_attribute_name = self.rename_if_field_name_is_keyword(model_attribute_name)
        model_attribute_name = self.trim_if_field_name_too_long(model_attribute_name)
        model_attribute_name = model_attribute_name.strip("_")
        model_attribute_name = self.add_number_suffix_if_duplicated_field_name(model_attribute_name)
        model_attribute_name = rename_if_field_name_starts_with_number(model_attribute_name)
        return model_attribute_name

    def rename_if_field_name_is_keyword(self, model_attribute_name):
        if model_attribute_name in self.reserved_keywords:
            model_attribute_name += "item_"
        return model_attribute_name

    def trim_if_field_name_too_long(self, model_attribute_name):
        if len(model_attribute_name) > self.MAX_FIELD_NAME_LENGTH:
            model_attribute_name = model_attribute_name[0:self.MAX_FIELD_NAME_LENGTH]
        return model_attribute_name

    def add_number_suffix_if_duplicated_field_name(self, model_attribute_name):
        if model_attribute_name in self.mapping_dict:
            model_attribute_name += "_%d" % random.randint(0, 999)
        return model_attribute_name

    def create_attr_list_code_lines(self):
        self.attr_list_code_lines.append("attr_order = [")
        for col in self.header:
            name = get_valid_excel_field_name(unicode(col))
            if not (name in self.attr_list):
                self.attr_list.append(name)
                self.attr_list_code_lines.append(self.indents + 'u"""%s""",' % name)

        self.attr_list_code_lines.append("]")

    def get_mapping_and_attr(self):
        res = [u"from django.db import models",
               u"from django.utils.translation import ugettext as _",
               u"from six import python_2_unicode_compatible",
               u"from djangoautoconf.model_utils.len_definitions import %s, TEXT_LENGTH_128" % self.field_len_definition,
               u"",
               u""]
        res += self.mapping_code_lines + ["", ""]
        res += self.attr_list_code_lines
        return "\n".join(res)

    def create_mapping_for_excel(self):
        self.create_mapping("mapping")
        self.create_attr_list_code_lines()
        return self.get_mapping_and_attr()


class ExcelFileProcessor(FolderFileProcessor):
    def __init__(self, files_folder=""):
        super(ExcelFileProcessor, self).__init__(files_folder)
        self.has_processed_once = False

    def is_need_process(self, file_path):
        return (not self.has_processed_once) and (not ("~" in file_path))

    def process_file(self, full_path):
        self.has_processed_once = True
        m = ModelCreator(full_path)
        print m.create_mapping_for_excel()
        print m.create_model()


def create_model_and_import_script():
    processor = ExcelFileProcessor(os.sep.join(["data", "dumping_excel_file"]))
    processor.process_files()


class Command(BaseCommand):
    help = 'Create model script from excel'

    def handle(self, *args, **options):
        create_model_and_import_script()



import os
import sys
from background_task import background
from models import ExcelImportTask
from django.contrib.auth.models import User


PROCESS_NUMBER_IN_ONE_TURN = 2000


@background(schedule=10)
def import_excel():
    # lookup user by id and send them a message
    # user = User.objects.get(pk=user_id)
    tasks = ExcelImportTask.objects.filter(is_completed=False)
    for task in tasks:
        # mapping_dict = task.content.model_class().__module__.mapping
        # attr_list = task.content.model_class().__module__.attr_list
        full_path = task.excel_file
        # next = task.next_process_line_numbered_from_1
        # task.content.model_class().__module__
        # task.content.pk
        cmd = "%s manage.py import_excel_according_to_model %s %d %d %d %d" % \
              (sys.executable, full_path, task.content.pk, task.header_row_numbered_from_1,
               task.next_process_line_numbered_from_1, PROCESS_NUMBER_IN_ONE_TURN)
        print 'executing: ', cmd
        res = os.system(cmd)
        print "execute result:", res
        if res != 0:
            task.is_completed = True
            # task.next_process_line_numbered_from_1 = 0
        else:
            task.next_process_line_numbered_from_1 += PROCESS_NUMBER_IN_ONE_TURN
            import_excel()
        task.save()




from django.conf.urls import patterns, url
from django_excel_to_model.views_import import upload_file
from views import create_mapping

urlpatterns = patterns('',
                       # (r'^translate/',
                       #  ExcelMappingGeneratorView.as_view(template_name=
                       #                                    'excel_reader/excel_mapping_generator.html')),
                       (r'^$', create_mapping),
                        url(r'^upload/', upload_file),
                       )



import os
import tempfile
from sys import platform

from django.core.management import call_command
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.views.decorators.csrf import csrf_exempt
from ufs_tools.string_tools import class_name_to_low_case

from django_excel_to_model.django_app_generator import DjangoAppGenerator
from django_excel_to_model.management.commands.dump_excel_to_mapping import ModelCreator
from forms import ExcelFormatTranslateForm


@csrf_exempt
def create_mapping(request):
    form = ExcelFormatTranslateForm(request.POST or None,
                                    request.FILES or None)
    context = {}
    data_start_row = 1

    if form.is_valid():
        import_file = form.cleaned_data['import_file']
        header_row_numbered_from_1 = form.cleaned_data['header_row_numbered_from_1']
        spreadsheet_numbered_from_1 = form.cleaned_data['spreadsheet_numbered_from_1']
        class_name = form.cleaned_data['class_name']
        app_name = class_name_to_low_case(class_name).replace("_", "-")

        # first always write the uploaded file to disk as it may be a
        # memory file or else based on settings upload handlers
        with tempfile.NamedTemporaryFile(delete=False) as uploaded_file:
            for chunk in import_file.chunks():
                uploaded_file.write(chunk)
            uploaded_file.close()
            m = ModelCreator(uploaded_file.name, header_row_numbered_from_1-1, spreadsheet_numbered_from_1-1, class_name)
            codes = m.create_mapping_for_excel()
            codes += "\n"  # Add an additional line to model file
            codes += m.create_model(data_start_row)

            context['codes'] = codes

            if form.cleaned_data["is_create_app_now"]:
                g = DjangoAppGenerator(app_name)
                g.create_default_structure()
                g.create_module_file("models.py", codes)

                if platform == "linux" or platform == "linux2":
                    # linux
                    os.system('python manage.py makemigrations')
                    os.system('python manage.py migrate')
                    os.system('touch manage.py')

    context['form'] = form
    return render_to_response("excel_reader/excel_mapping_generator.html",
                              context, context_instance=RequestContext(request))



from django.contrib.contenttypes.models import ContentType
from django.utils.translation import ugettext_lazy as _
from django import forms
from models import ExcelImportTask
from django.forms import ModelForm


class ExcelFormatTranslateForm(forms.Form):
    # title = forms.CharField(max_length=50)
    import_file = forms.FileField(
        label=_('File to import')
    )
    header_row_numbered_from_1 = forms.IntegerField()
    spreadsheet_numbered_from_1 = forms.IntegerField()
    class_name = forms.CharField()
    is_create_app_now = forms.BooleanField(required=False)


class ExcelImportTaskForm(ModelForm):
    content = forms.ModelChoiceField(queryset=ContentType.objects.order_by('model'))

    class Meta:
        model = ExcelImportTask
        fields = ['excel_file', 'content', "header_row_numbered_from_1", "spreadsheet_numbered_from_1"]

    is_import_now = forms.BooleanField(required=False)
    is_clean_before_import = forms.BooleanField(required=False)



from django_excel_to_model.field_tools import get_target_field_name
from django_excel_to_model.management.commands.dump_excel_to_mapping import rename_if_field_name_starts_with_number


class ClassAttributeCreator(object):
    reserved_keywords = ["type", "class", "for", "in", "while"]
    MAX_FIELD_NAME_LENGTH = (64-5)

    def rename_if_field_name_is_keyword(self, model_attribute_name):
        if model_attribute_name in self.reserved_keywords:
            model_attribute_name += "item_"
        return model_attribute_name

    def trim_if_field_name_too_long(self, model_attribute_name):
        if len(model_attribute_name) > self.MAX_FIELD_NAME_LENGTH:
            model_attribute_name = model_attribute_name[0:self.MAX_FIELD_NAME_LENGTH]
        return model_attribute_name

    def refine_attr_name(self, attr_name):
        model_attribute_name = get_target_field_name(attr_name).lower()
        model_attribute_name = self.rename_if_field_name_is_keyword(model_attribute_name)
        model_attribute_name = self.trim_if_field_name_too_long(model_attribute_name)
        model_attribute_name = model_attribute_name.strip("_")
        # model_attribute_name = self.add_number_suffix_if_duplicated_field_name(model_attribute_name)
        model_attribute_name = rename_if_field_name_starts_with_number(model_attribute_name)
        return model_attribute_name



import argparse
import importlib
import os
from django.contrib.contenttypes.models import ContentType
from django.core.management.base import BaseCommand
from django_excel_to_model.reader import ExcelFile, XlsbFile
# from ufs_tools.inspect_utils import class_enumerator


class DictTranslator(object):
    # noinspection PyMethodMayBeStatic
    def translate(self, item_dict):
        for ignored in ["__invalid"]:
            if ignored in item_dict:
                del item_dict["__invalid"]
        return item_dict


class ExcelFileFromClassImporter(object):
    def __init__(self, class_instance, sheet_numbered_from_1=1):
        super(ExcelFileFromClassImporter, self).__init__()
        self.model_module = importlib.import_module(class_instance.__module__)
        self.class_instance = class_instance
        self.translator = DictTranslator()
        self.sheet_numbered_from_1 = sheet_numbered_from_1

    def import_excel(self, full_path, header_row_numbered_from_1, first_import_row_numbered_from_1=1, count=1000):
        if 'xlsb' in full_path:
            excel_file = XlsbFile(full_path)
        elif 'xlsx' in full_path:
            excel_file = ExcelFile(full_path)
        filename = os.path.basename(full_path)
        sheet = excel_file.get_sheet(self.sheet_numbered_from_1-1)
        sheet.init_header_raw(header_row_numbered_from_1 - 1)
        # for class_instance in class_enumerator(self.model_module):
        #     new_item_class = class_instance
        cnt = 0

        for item_info_dict in sheet.enumerate_mapped(self.model_module.mapping,
                                                     start_row=first_import_row_numbered_from_1):
            print item_info_dict
            self.translator.translate(item_info_dict)
            item_info_dict["data_import_id"] = filename
            target, is_created = self.class_instance.objects.get_or_create(**item_info_dict)
            if not is_created:
                print "item duplicated:", item_info_dict
            cnt += 1
            # If count = 1, when 1 processed, cnt will become 1
            if cnt > count:
                return 0
        return -1


def import_excel_according_to_model(full_path, content_type_id, header_row_numbered_from_1,
                                    first_import_row_numbered_from_1, count=1000):
    content = ContentType.objects.get(pk=content_type_id)
    e = ExcelFileFromClassImporter(content.model_class())
    e.import_excel(full_path, header_row_numbered_from_1, first_import_row_numbered_from_1, count)


class Command(BaseCommand):
    help = 'Import excel according to model info'

    def add_arguments(self, parser):
        parser.add_argument('file-path')
        parser.add_argument('content-type-id')
        parser.add_argument('header_row_numbered_from_1')
        parser.add_argument('start')
        parser.add_argument('count', nargs='?')

    def handle(self, *args, **options):
        parser = argparse.ArgumentParser(description='Import excel file according to model info')

        subparsers = parser.add_subparsers(help='sub-command help')

        # create the parser for the "a" command
        parser_import_excel_according_to_model = subparsers.add_parser(
            'import_excel_according_to_model', help='import_excel_according_to_model help')

        # parser_import_excel_according_to_model.add_argument(
        #     '--content-type-id', type=int, help='content type pk')
        # parser_import_excel_according_to_model.add_argument(
        #     '--start', default=1,
        #     help='start line for importing excel, default=1 (second line)')
        parser_import_excel_according_to_model.add_argument(
            'file-path', nargs=1, help='path of the excel file')
        parser_import_excel_according_to_model.add_argument(
            'content-type-id', nargs=1, help='content id of the model', type=int)
        parser_import_excel_according_to_model.add_argument(
            'header_row_numbered_from_1', nargs=1, help='header row number (start from 1)', type=int)
        parser_import_excel_according_to_model.add_argument(
            'start', nargs=1, help='path of the excel file', type=int)
        parser_import_excel_according_to_model.add_argument(
            'count', nargs="?", help='process line count', default=[1000], type=int)

        arg_result = parser.parse_args()
        print vars(arg_result)["file-path"][0]
        print vars(arg_result)["content-type-id"][0]
        print vars(arg_result)["start"][0]
        print vars(arg_result)["count"]

        file_path = vars(arg_result)["file-path"][0]
        content_type_id = vars(arg_result)["content-type-id"][0]
        header_row_numbered_from_1 = vars(arg_result)["header_row_numbered_from_1"][0]
        first_import_row_numbered_from_1 = vars(arg_result)["start"][0]
        count = vars(arg_result)["count"]

        return import_excel_according_to_model(
            file_path, content_type_id, header_row_numbered_from_1, first_import_row_numbered_from_1, count)



import pinyin


def get_target_field_name(col):
    col = pinyin.get(col, format="strip")
    for ch in [" ", ",", "_", ")", "(", ":", "/", "\\", '"', "'", "-", ",", ".", "<", ">", "%", "&", "\r", "\n"]:
        col = col.replace(ch, "_").replace("__", "_")
    # col = filter(lambda x: x in string.printable, col)
    col = get_string_with_only_char_in_list(col)
    col = col.replace("__", "_")
    col = col.strip("_")
    if col == "id":
        return "original_id"
    if col == "global":
        return "global_item"
    return col


var_name_char_list = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ_"


def get_string_with_only_char_in_list(col, char_list=var_name_char_list):
    res = u""
    for ch in col:
        if ch in char_list:
            res += ch
    col = res
    return col


def get_valid_excel_field_name(col):
    """
    This function will not return the exact string!!!!!!
    :param col:
    :return:
    """
    if not (type(col) is unicode):
        raise "Non-unicode not supported"
    # Only ascii supported here
    return get_string_with_only_char_in_list(col, var_name_char_list + " \t")
    #
    # for ch in [u"'", u'"']:
    #     col = col.replace(ch, u"\\"+ch)
    #
    # # col = col.encode("utf8", errors="ignore")
    # # col = col.decode("utf8")
    # # for ch in [u"\r", u"\n"]:
    # #     col = col.replace(ch, "")
    # # col = col.replace(u"\xa0\xa0", "")
    # # col = unicode(col.encode("ascii", errors="ignore"))
    # return col



import os

from ufs_tools.app_tools import get_executable_folder
from ufs_tools.folder_tool import ensure_dir


class DjangoAppGenerator(object):
    def __init__(self, app_name):
        super(DjangoAppGenerator, self).__init__()
        self.app_name = app_name
        self.app_module_name = app_name.replace("-", "_")
        self.module_path = None
        self.admin_file_content = '''import models
from djangoautoconf.auto_conf_admin_tools.admin_register import AdminRegister
factory = AdminRegister()
factory.register_all_model(models)
'''
        self.urls_content = '''from djangoautoconf.model_utils.url_for_models import add_all_urls
from django.conf.urls import patterns
import models

urlpatterns = patterns('',)

add_all_urls(urlpatterns, models)
'''

    def create_default_structure(self):
        repo_root = os.path.abspath(get_executable_folder())
        target_repo_folder = os.path.join(repo_root, "external_app_repos/auto-generated-apps")
        target_app_folder = os.path.join(target_repo_folder, self.app_name)
        ensure_dir(target_repo_folder)
        if os.path.exists(target_app_folder):
            raise "Folder exists"
        else:
            os.mkdir(target_app_folder)
            self.module_path = os.path.join(target_app_folder, self.app_module_name)
            os.mkdir(self.module_path)
            os.mkdir(os.path.join(self.module_path, "migrations"))

            self.create_module_file("admin.py", self.admin_file_content)
            self.create_module_file("urls.py", self.urls_content)
            self.create_module_file("default_settings.py")
            self.create_module_file("__init__.py")
            self.create_module_file("migrations/__init__.py")

    def create_module_file(self, target_file, file_content=""):
        admin_file_path = os.path.join(self.module_path, target_file)
        admin_file = open(admin_file_path, "w")
        admin_file.write(file_content)
        admin_file.close()



# -*- coding: utf-8 -*-
from datetime import datetime, tzinfo
import pytz
import pyxlsb
from xlrd import open_workbook, cellname, XL_CELL_DATE, xldate_as_tuple, XL_CELL_NUMBER
from django.utils import timezone
from field_tools import get_valid_excel_field_name


class Excel(object):
    def __init__(self, full_path):
        self.workbook = open_workbook(full_path, formatting_info=True)

    def get_sheet(self, sheet_name):
        return self.workbook.sheet_by_name(sheet_name)

    def get_sheet_from_index(self, index):
        return self.workbook.sheet_by_index(index)

    def read_sheet(self, sheet_name):
        pass


class BaseSheet(object):
    def __init__(self, workbook, index):
        self.workbook = workbook
        self.sheet = None
        self.title_columns = None

    def format_number(self, cell):
        print 'format number'

    def parse_cell_value(self, cell):
        print 'get cell value according to cell type'

    def get_mapped_column(self, row_index, mapping):
        print 'get mapped column'

    def enumerate_mapped(self, mapping, start_row=1):
        print 'enumerate mapped column'

    def init_header_raw(self, header_row_index=0):
        print 'init title column'

    def get_columns(self, row_index):
        print 'get column'


class XlsbSheet(BaseSheet):
    def __init__(self, workbook, index=0):
        self.workbook = workbook
        self.sheet = workbook.get_sheet(index+1)
        self.title_columns = None

    def format_number(self, cell):
        if cell.v == int(cell.v):
            return int(cell.v)
        return cell.v

    def parse_cell_value(self, cell):
        try:
            final_value = int(cell.v)
        except ValueError:
            final_value = cell.v
        return final_value

    def get_mapped_columns(self, row_index, mapping):
        res = {}
        for row in self.sheet.rows(sparse=True):
            for r in row:
                if r.v is not None and r.r== row_index:
                    src_key = unicode(self.title_columns[r.c])
                    try:
                        target_key = mapping[src_key]
                    except KeyError:
                        target_key = mapping[get_valid_excel_field_name(src_key)]
                    cell = r
                    res[target_key] = self.parse_cell_value(cell)
        return res

    def enumerate_mapped(self, mapping, start_row=1):
        for row in self.sheet.rows(sparse=True):
            res = {}
            for r in row:
                if r.v is not None and r.v not in self.title_columns:
                    if r.r % 1000 == 0:
                        print r, r.r
                    src_key = unicode(self.title_columns[r.c])
                    try:
                        target_key = mapping[src_key]
                    except KeyError:
                        target_key = mapping[get_valid_excel_field_name(src_key)]
                    res[target_key] = self.parse_cell_value(r)
            yield res

    def init_header_raw(self, header_row_index=0):
        self.title_columns = self.get_columns(header_row_index)
        return self.title_columns

    def get_columns(self, row_index):
        res = []
        for row in self.sheet.rows(sparse=True):
            for r in row:
                if r.r == row_index and r.v is not None:
                    res.append((str(r.v)))
        return res


class Sheet(BaseSheet):
    def __init__(self, workbook, index = 0):
        self.workbook = workbook
        self.sheet = workbook.sheet_by_index(index)
        self.title_columns = None
        self.formatting_info = True

    def disable_formatting_info(self):
        self.formatting_info = False

    def get_columns(self, row_index):
        res = []
        for col_index in range(self.sheet.ncols):
            # print cellname(row_index, col_index), '-',
            # print self.sheet.cell(row_index, col_index).value
            res.append(self.sheet.cell(row_index, col_index).value)
        return res

    def get_mapped_columns(self, row_index, mapping):
        res = {}
        for col_index in range(self.sheet.ncols):
            # print self.sheet.cell(row_index, col_index).value
            src_key = unicode(self.title_columns[col_index])
            # print "___________________", src_key
            try:
                target_key = mapping[src_key]
            except KeyError:
                target_key = mapping[get_valid_excel_field_name(src_key)]
            cell = self.sheet.cell(row_index, col_index)
            res[target_key] = self.parse_cell_value(cell)
        return res

    def format_number(self, cell):
        if self.formatting_info:
            xf = self.workbook.xf_list[cell.xf_index]
            fmt_key = xf.format_key
            fmt = self.workbook.format_map[fmt_key]
            if fmt.format_key == 0:
                # Its format is General, so no additional fractional part should be displayed
                if cell.value == float(int(cell.value)):
                    return int(cell.value)
        elif cell.value == int(cell.value):
            return int(cell.value)
        return cell.value

    def get_format_info(self, cell):
        print "cell.xf_index is", cell.xf_index
        fmt = self.workbook.xf_list[cell.xf_index]
        print "type(fmt) is", type(fmt)
        print
        print "fmt.dump():"
        fmt.dump()

    def enumerate_raw(self):
        start_row = 1
        for row_index in range(start_row, self.sheet.nrows):
            row_data = []
            for col_index in range(self.sheet.ncols):
                # print self.sheet.cell(row_index, col_index).value
                src_key = unicode(self.title_columns[col_index])
                # print "___________________", src_key
                row_data.append(self.sheet.cell(row_index, col_index))
            yield row_data

    def get_total_rows(self):
        return self.sheet.nrows

    def enumerate_mapped(self, mapping, start_row=1):
        for self.row_index in range(start_row, self.sheet.nrows):
            # sheet_columns = self.sheet.ncols
            if self.row_index % 1000 == 0:
                print self.sheet.nrows, self.row_index
            yield self.get_mapped_columns(self.row_index, mapping)

    def get_current_row_index(self):
        return self.row_index

    def enumerate(self, start_row=1):
        for row_index in range(start_row, self.sheet.nrows):
            sheet_columns = self.sheet.ncols
            print self.sheet.nrows, row_index
            yield self.get_header_to_column_dict(row_index)

    def parse_cell_value(self, cell):
        if cell.ctype == XL_CELL_DATE:
            date_for_cell = xldate_as_tuple(cell.value, 0)
            native_date = datetime(date_for_cell[0], date_for_cell[1], date_for_cell[2], date_for_cell[3],
                                   date_for_cell[4], date_for_cell[5])
            try:
                time_zone_aware = timezone.make_aware(native_date, timezone.get_current_timezone())
            except OverflowError:
                time_zone_aware = native_date.replace(tzinfo=pytz.UTC)
            final_value = time_zone_aware
        elif cell.ctype == XL_CELL_NUMBER:
            # self.get_format_info(cell)
            final_value = self.format_number(cell)
        else:
            final_value = cell.value
        return final_value

    def get_header_to_column_dict(self, row_index):
        res = {}
        for col_index in range(self.sheet.ncols):
            # print self.sheet.cell(row_index, col_index).value
            src_key = unicode(self.title_columns[col_index])
            # print "___________________", src_key
            cell = self.sheet.cell(row_index, col_index)
            res[src_key] = self.parse_cell_value(cell)
        return res

    def init_header_raw(self, header_row_index=0):
        self.title_columns = self.get_columns(header_row_index)
        return self.title_columns

    def get_headers(self):
        self.init_header_raw()
        for column in self.title_columns:
            print column,
        print ""

    def get_title_columns(self):
        return self.title_columns


class SheetConfig(object):
    def __init__(self):
        self.__has_header = True

    def has_header(self):
        return self.__has_header


class ExcelBaseFile(object):
    def __init__(self, full_path):
        raise "Failed to get open workbook"

    def get_sheet(self, index):
        raise "Failed to get sheet"


class ExcelFile(ExcelBaseFile):
    def __init__(self, full_path):
        self.workbook = open_workbook(full_path)

    def get_sheet(self, index):
        s = Sheet(self.workbook, index)
        s.disable_formatting_info()
        return s


class XlsbFile(ExcelBaseFile):
    def __init__(self, full_path):
        self.workbook = pyxlsb.open_workbook(full_path)

    def get_sheet(self, index):
        sheet = XlsbSheet(self.workbook, index)
        return sheet





# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='ExcelImportTask',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('excel_file', models.FileField(upload_to=b'data/excel_import_task_files/')),
                ('header_row_numbered_from_1', models.IntegerField(default=1)),
                ('next_process_line_numbered_from_1', models.IntegerField(default=1)),
                ('spreadsheet_numbered_from_1', models.IntegerField(default=1)),
                ('is_completed', models.BooleanField(default=False)),
                ('content', models.ForeignKey(to='contenttypes.ContentType')),
            ],
        ),
    ]



from django_excel_to_model.management.commands.model_create_utils.attribute_generator import ClassAttributeCreator
import django_tables2 as tables


def get_django_tables2_from_dict(data_dict):
    c = ClassAttributeCreator()
    table_meta_class = type("Meta", (), {
        "attrs": {'class': 'table table-striped table-bordered table-advance table-hover'},
        "orderable": False,
    })
    table_attributes = {"Meta": table_meta_class}
    table_data = {}
    for data_key in data_dict.keys():
        attr_name = c.refine_attr_name(data_key)
        table_attributes[attr_name] = tables.Column(verbose_name=data_key)
        table_data[attr_name] = data_dict[data_key]

    item_table_class = type("Item" + "DictTableClass", (tables.Table,), table_attributes)

    return item_table_class([table_data])



# -*- coding: utf-8 -*-
from django.contrib.contenttypes.models import ContentType
from django.db import models


class ExcelImportTask(models.Model):
    excel_file = models.FileField(upload_to="data/excel_import_task_files/")
    header_row_numbered_from_1 = models.IntegerField(default=1,  # help_text="Header row number (start from 1)")
                                                     )
    next_process_line_numbered_from_1 = models.IntegerField(default=1)
    spreadsheet_numbered_from_1 = models.IntegerField(default=1)
    is_completed = models.BooleanField(default=False)
    content = models.ForeignKey(ContentType)



from openpyxl import Workbook


class ExcelFile(object):
    def __init__(self, full_path, flag):
        super(ExcelFile, self).__init__()
        self.excel_file = Workbook(guess_types=True)

    def get_sheet(self, index):
        return self

    def get_header_raw(self, index):
        ws = self.excel_file.get_active_sheet()

        for row in ws.iter_rows():
            for cell in row:
                print cell
            break